breed[divisions division]
patches-own[terrain water]
divisions-own[nation tech reliability manpower maxmanpower organization maxOrganization recovering]
globals [temperature nazi-deaths soviet-deaths precipitation]

;; Setup Functions

to setup
  clear-all
  set-default-shape divisions "square"
  set temperature 16.1
  set nazi-deaths 0
  set soviet-deaths 0
  set precipitation 0
  setup-patches
  setup-soviets
  place-soviets
  setup-nazis
  place-nazis
  reset-ticks
end

to setup-nazis
  ;;49 divisions in total for army group center
  create-divisions 49 [setup-nazi-division]
end

to setup-soviets
  ;;45 divisions in total for western front
  create-divisions 45 [setup-soviet-division]
end

to setup-soviet-division

  set heading 270
  set color red + 3
  set manpower (random 4000) + 8000
  set maxmanpower manpower
  set tech random 30
  set reliability 10
  set organization (random 30) + 15
  set maxOrganization organization
  set nation "soviet"
  set recovering false
end

to place-soviet-division
  set xcor random 3 - 20
  set ycor random 20 - 10
end

to place-soviets
  ask divisions with [nation = "soviet"] [
   place-soviet-division
  ]
end

to setup-nazi-division
  set heading 90
  set color grey + 3
  set manpower (random 2000) + 10000
  set maxmanpower manpower
  set tech (random 30) + 10
  set reliability 20
  set organization (random 30) + 30
  set maxOrganization organization
  set nation "nazi"
  set recovering false
end

to place-nazi-division
  set xcor random 3 - 23
  set ycor random 20 - 10
end

to place-nazis
  ask divisions with [nation = "nazi"] [
   place-nazi-division
  ]
end

to setup-patches
  ask patches[
    set water 0
    setup-patch]
end

to setup-patch
  ;;for now just set them all to 0
  set terrain (1 / 10 * pxcor) - ((random 2) - 1)
  ifelse pxcor < -20 [set pcolor grey + terrain]
  [set pcolor red + terrain]
end












;; Simulation functions

to go
  set temperature (11.05 * (cos ticks) + 7.05)
  set temperature temperature + random-float 12 - 6
  ifelse temperature > 20 [set precipitation .7]
  [ifelse temperature > 15 [set precipitation .3]
    [ifelse temperature > 5 [set precipitation .6]
    [set precipitation .2]]]
  if ticks > 180 [stop]
  move-nazi-divisions
  move-soviet-divisions
  ask divisions [
   set organization organization + 1
   set manpower manpower + 100
   if manpower >= maxmanpower [set manpower maxmanpower]
  ]
  if remainder ticks soviet-rate = 0 [reinforce-soviets]
  if remainder ticks nazi-rate = 0 [reinforce-nazis]
  ask patches [simulate-rain]
  diffuse water .2
  tick
end

to move-soviet-division
  ifelse (not any? (divisions-on [neighbors] of patch-here) with [nation = "nazi"])[
    ifelse any? ([neighbors] of patch-here) with [shade-of? grey pcolor][
      move-to one-of ([neighbors] of patch-here) with [shade-of? grey pcolor]
      ask patch-here [set pcolor red + terrain]
      ]
    [
      ifelse can-move? 1 [fd 1]
      [move-to one-of [neighbors] of patch-here]
      ]
    ]
  []
end

to move-soviet-divisions
  ask divisions with [nation = "soviet"] [
   ifelse not recovering [
     move-soviet-division
   ]
    [if organization >= maxOrganization [
        set organization maxOrganization
        set recovering false]]
  ]
end

to move-nazi-division
  ifelse any? ((divisions-on ([neighbors4] of patch-here)) with [nation = "soviet"]) [
    ifelse organization < 0 or recovering []
    [battle one-of other (divisions-on ([neighbors4] of patch-here)) with [nation = "soviet"]]
  ]
  [ifelse any? ([neighbors] of patch-here) with [shade-of? red pcolor] [
    move-to one-of ([neighbors] of patch-here) with [shade-of? red pcolor]
    ask patch-here [set pcolor grey + terrain]]
  [ifelse can-move? 1 [
      fd 1][
      move-to one-of [neighbors] of patch-here]]]
end

to move-nazi-divisions
  ask divisions with [nation = "nazi"] [
    ifelse not recovering [
      if [water] of patch-here < 1 [
        move-nazi-division
      ]
    ]
    [if organization >= maxOrganization [
        set organization maxOrganization
        set recovering false]]
  ]
end

to battle [#enemy]
  ;;Our attack

  ask #enemy [
    let attack-strength random (([tech] of myself) + .1 * [manpower] of myself)
    set attack-strength attack-strength - ([terrain] of patch-here) * 20 * terrain-scale

    let reliability-modifier [temperature / reliability] of myself
    if reliability-modifier > 1 [set reliability-modifier 1]
    if reliability-modifier < 0 [set reliability-modifier .1]
    set attack-strength attack-strength * reliability-modifier

    if attack-strength < 0 [set attack-strength 0]
    set manpower manpower - attack-strength

    ifelse [nation = "nazi"] of myself
    [set soviet-deaths soviet-deaths + attack-strength]
    [set nazi-deaths nazi-deaths + attack-strength]

    set organization organization - random-float .01 * attack-strength
    if organization <= 0
    [
     set organization 0
     set recovering true
     fd -4
    ]
    if manpower < 0 [
      set manpower 0
      die
    ]
  ]

  if #enemy != nobody [

  ;;Enemy counter-attack
  let attack-strength random (([tech] of #enemy) + .1 * [manpower] of #enemy)
  set attack-strength attack-strength - ([terrain] of patch-here) * 20 * terrain-scale

  let reliability-modifier [temperature / reliability] of #enemy
  if reliability-modifier > 1 [set reliability-modifier 1]
  if reliability-modifier < 0 [set reliability-modifier .1]
  set attack-strength attack-strength * reliability-modifier

  if attack-strength < 0 [set attack-strength 0]
  set manpower manpower - attack-strength
  ifelse nation = "nazi"
  [set nazi-deaths nazi-deaths + attack-strength]
  [set soviet-deaths soviet-deaths + attack-strength]
  set organization organization - random-float .01 * attack-strength
    if organization <= 0 [
      set organization 0
      set recovering true
      fd -4
    ]
    if manpower < 0 [
      set manpower 0
      die
    ]]
end

to reinforce-soviets
  if any? patches with [shade-of? red pcolor and pxcor > 25][
    create-divisions soviet-reinforcements [
      setup-soviet-division
      move-to one-of patches with [shade-of? red pcolor and pxcor > 25]]]
end

to reinforce-nazis
  if any? patches with [shade-of? grey pcolor and pxcor < -23][
    create-divisions nazi-reinforcements [
      setup-nazi-division
      move-to one-of patches with [shade-of? grey pcolor and pxcor < -23]]]
end

to simulate-rain
  if random-float 1 < precipitation [
   set water water + random 4
  ]
  set water water - 1
  if water < 0 [set water 0]
end
@#$#@#$#@
GRAPHICS-WINDOW
640
48
1615
411
30
10
15.824
1
10
1
1
1
0
0
0
1
-30
30
-10
10
0
0
1
ticks
30.0

BUTTON
51
80
114
113
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
116
79
179
112
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

PLOT
67
190
444
489
Army Strengths
Time
Army Strength
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"soviets" 1.0 0 -2674135 true "" "plot sum ([manpower] of (divisions with [nation = \"soviet\"]))"
"nazis" 1.0 0 -7500403 true "" "plot sum ([manpower] of (divisions with [nation = \"nazi\"]))"

SLIDER
196
77
368
110
terrain-scale
terrain-scale
0
10
1
1
1
NIL
HORIZONTAL

PLOT
67
491
444
786
Daily Temperature
Time
Temperature (C)
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -14070903 true "" "plot temperature"

PLOT
445
491
826
786
Army Casualties
Time
Casualties
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "plot soviet-deaths"
"pen-1" 1.0 0 -7500403 true "" "plot nazi-deaths"

SLIDER
196
45
368
78
weather-scale
weather-scale
0
10
1
1
1
NIL
HORIZONTAL

SLIDER
411
49
583
82
soviet-reinforcements
soviet-reinforcements
0
15
2
1
1
NIL
HORIZONTAL

SLIDER
411
81
583
114
nazi-reinforcements
nazi-reinforcements
0
10
1
1
1
NIL
HORIZONTAL

SLIDER
411
115
583
148
soviet-rate
soviet-rate
0
21
12
1
1
NIL
HORIZONTAL

SLIDER
410
148
582
181
nazi-rate
nazi-rate
0
21
12
1
1
NIL
HORIZONTAL

PLOT
842
492
1181
784
Impassability of Terrain
Water
Time
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -6459832 true "" "plot sum [water] of patches"

@#$#@#$#@
## WHAT IS IT?
This model attempts to explore the effects of weather and terrain on the progression of Operation Barbarossa and Operation Typhoon, two major offensives that were crucial in determining the outcome of world war 2.

## HOW IT WORKS

Each patch represents an area of terrain and each turtle represents a division.  Grey turtles attack and take red territory, and red turtles defend and take grey territory.  Weather is also simulated which effects the fighting capabilities of each side differently.

## HOW TO USE IT

You can adjust the effects of weather and terrain by adjusting their sliders, higher values mean they have more of an impact during battles.  The reinforcement rates for each side can also be adjusted to change how frequently and by how much each side is reinforced.

## THINGS TO NOTICE
Notice the slowdown once the temperature drops below 10 degrees celcius, as battles take longer to resolve.

## THINGS TO TRY


Try running the model with no terrain or weather effects to see what the Germans expected to happen, and then crank them up to see how important they are when thinking about how easily it really was.

## EXTENDING THE MODEL

Try finding more accurate historical data to improve the battle simulation, or a levelspace simlulation of each battle at the regimental level.  Also try to find the historical rate for reinforcement for each side.

## NETLOGO FEATURES

This model does not use any unusual features of Netlogo

## RELATED MODELS

This is the first model I have seen which attemtps to model a military campaign.

## CREDITS AND REFERENCES
Check out this model and many others at http://modelingcommons.org/browse/one_model/4652#model_tabs_browse_nlw
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 6.0-M5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="Operation Barbarossa" repetitions="5" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>sum [manpower] of divisions with [nation = "soviet"]</metric>
    <metric>sum [manpower] of divisions with [nation = "nazi"]</metric>
    <metric>soviet-deaths</metric>
    <metric>nazi-deaths</metric>
    <enumeratedValueSet variable="soviet-reinforcements">
      <value value="2"/>
    </enumeratedValueSet>
    <steppedValueSet variable="weather-scale" first="0" step="1" last="10"/>
    <steppedValueSet variable="soviet-rate" first="10" step="1" last="12"/>
    <steppedValueSet variable="nazi-rate" first="10" step="1" last="12"/>
    <steppedValueSet variable="terrain-scale" first="0" step="1" last="10"/>
    <enumeratedValueSet variable="nazi-reinforcements">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
